import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Oregon raw data for consolidation."""

    postal_code = "OR"
    fields = dict(
        company="Company Name",
        location="Location",
        notice_date="Received Date",
        effective_date="Layoff Date",
        jobs="Laid Off",
    )
    date_format = "%Y-%m-%d %H:%M:%S"
    minimum_year = 1988
    jobs_corrections = {
        # This is a nationwide Northwest Airlines layoff that is large and legit.
        # https://www.nytimes.com/1998/09/03/us/northwest-lays-off-27000-increasing-pressure-on-strike.html
        27500: 27500,
    }
    date_corrections = {"1899-12-29 00:00:00": None}

    def check_if_temporary(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a temporary or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "temporary" in row["Layoff Type"].lower() or None

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "closure" in row["Layoff Type"].lower() or None



import os
import sys
from datetime import datetime

# Insert the parent directory into the path
sys.path.insert(0, os.path.abspath(".."))
sys.path.insert(0, os.path.abspath("./"))

extensions = [
    "myst_parser",
    "sphinx.ext.autodoc",
    "sphinxcontrib.napoleon",
    "sphinx_click",
]
templates_path = ["_templates"]
source_suffix = {
    ".rst": "restructuredtext",
    ".md": "markdown",
}
master_doc = "index"

project = "warn-transformer"
year = datetime.now().year
copyright = f"{year} Big Local News"

exclude_patterns = ["_build"]

pygments_style = "sphinx"



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Texas raw data for consolidation."""

    postal_code = "TX"
    fields = dict(
        company="JOB_SITE_NAME",
        location="CITY_NAME",
        notice_date="NOTICE_DATE",
        effective_date="LayOff_Date",
        jobs="TOTAL_LAYOFF_NUMBER",
    )
    date_format = "%Y-%m-%d %H:%M:%S"
    date_corrections = {
        "1930-03-30 00:00:00": datetime(2020, 3, 30),
        "1930-03-31 00:00:00": datetime(2020, 3, 31),
        "2027-03-01 00:00:00": datetime(2017, 3, 1),
    }



import pytest


def pytest_addoption(parser):
    """Add an argument to run the download and integrate tests."""
    parser.addoption("--runvcr", action="store_true", default=False, help="run VCR")


def pytest_configure(config):
    """Marker to tell VCR tests to run."""
    config.addinivalue_line("markers", "runvcr: mark test to run vcr")


def pytest_collection_modifyitems(config, items):
    """Ensure tests only run if --runvcr flag is supplied."""
    if config.getoption("--runvcr"):
        # --runvcr given in cli: will run vcr tests
        return
    run_vcr = pytest.mark.skip(reason="need --runvcr option to run")
    for item in items:
        if "runvcr" in item.keywords:
            item.add_marker(run_vcr)


@pytest.fixture(scope="module")
def vcr_config():
    """Replace API token with dummy placeholder."""
    return {"filter_headers": [("authorization", "DUMMY")]}



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Washington D.C. raw data for consolidation."""

    postal_code = "DC"
    fields = dict(
        company="Organization Name",
        location=lambda x: "Washington D.C.",  # Hardcode in the city name
        notice_date="Notice Date",
        effective_date="Effective Layoff Date",
        jobs="Number toEmployees Affected",
    )
    date_format = ("%B %d, %Y", "%m/%d/%y", "%B, %d, %Y", "%B %d,%Y")
    date_corrections = {
        "May 2 and 5, 2020": datetime(2020, 5, 2),
        "March, 20, 2020": datetime(2020, 3, 20),
        "31, 2019": datetime(2019, 12, 31),
        "August 2013": datetime(2013, 8, 1),
        "October 2013": datetime(2013, 10, 1),
        "May 7,14 & 31, 2012": datetime(2012, 5, 7),
        "February 28, 2022 March 31, 2022": datetime(2020, 2, 28),
        "December 25, and Feb - Jun 2021": datetime(2020, 12, 25),
        "TBD": None,
        "September 15, 2020 and March 18, 2020": datetime(2020, 9, 15),
        "May 31, 2012 June 15, 2012": datetime(2012, 5, 31),
        "June 29, 2012 & August 3, 2012": datetime(2012, 6, 29),
        "November 15 - December 16, 2022": datetime(2022, 11, 15),
        "December 3 - December 17, 2022": datetime(2022, 12, 3),
    }
    jobs_corrections = {
        "All": None,
        "TBD": None,
    }



import re
import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Ohio raw data for consolidation."""

    postal_code = "OH"
    fields = dict(
        company="Company",
        location="City/County",
        notice_date="Date Received",
        effective_date="Layoff Date(s)",
        jobs="Potential Number Affected",
    )
    date_format = "%m/%d/%Y"
    date_corrections = {
        "08/14/02018": datetime(2018, 8, 14),
        "01/30/201 7": datetime(2017, 1, 30),
        "10/30/20015": datetime(2015, 10, 30),
        "None": None,
        "Unknown": None,
        "10/2015": datetime(2015, 10, 1),
        "Various": None,
        "Mar‐16": None,
        "12/23/2015â": datetime(2015, 12, 23),
        "01/152024": datetime(2024, 1, 15),
        "3/5/202403/19/2024": datetime(2024, 3, 5),
        "3/5/2024-03/19/2024": datetime(2024, 3, 5),
        "10/31/2024;": datetime(2024, 10, 31),
    }
    jobs_corrections = {
        "13 FT": 13,
        "58 94 97 35": 58,
        "Unknown": None,
        "unknown": None,
        "242 80": 242,
        "323‐500": 323,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        # Cut out cruft
        value = value.replace("Updated", "")
        value = value.replace("Revised", "")
        value = value.replace("-", "").strip()

        # Split double dates
        if len(value) == 20:
            value = value[:10]
        elif len(value) == 19:
            value = value[:9]
        value = re.split(r"\s{2,}", value)[0].strip()
        value = value.split("Originated")[0].strip()
        # print(value)

        try:
            return super().transform_date(value)
        except Exception:
            value = value.split(" to ")[0].strip()
            value = value.split()[0].strip()
            value = value.replace("‐", "")
            return super().transform_date(value)



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Connecticut raw data for consolidation."""

    postal_code = "CT"
    fields = dict(
        company="affected_company",
        location="layoff_location",
        notice_date="warn_date",
        effective_date="layoff_date",
        jobs="number_workers",
    )
    date_format = ["%m/%d/%Y", "%m/%d/%y", "%m-%d-%y", "%Y"]
    date_corrections = {
        "12/31/16-1/13/17": datetime(2016, 12, 31),
        "12/4/15-tbd": datetime(2015, 12, 4),
        "2/29/15": datetime(2015, 2, 28),
        "9/1//15": datetime(2015, 9, 1),
        "Not Indicated": None,
        "# 12 2/13/17 through 2018": datetime(2017, 2, 13),
        "Most 10/29/16": datetime(2016, 10, 29),
        "Several Weeks - Five Months": None,
        "Not Dated Rec'd 6/24/15": datetime(2015, 6, 24),
        "3rd Quarter 2015-4th Quarter 2016": datetime(2015, 6, 1),
        "June 2017 - March 2018": datetime(2017, 6, 1),
        "First quarter 2019 - 2020": datetime(2019, 1, 1),
        "June 2018 - September 2, 2018": datetime(2018, 6, 1),
        "Beginning June 2018": datetime(2018, 6, 1),
        "December 2018 - March 1, 2019": datetime(2018, 12, 1),
        "Possibly 50+": None,
        "N/A": None,
        "Reduction in Hours Since March 2020": datetime(2020, 3, 1),
        "April-June 2020": datetime(2020, 4, 1),
        "": None,
        "7/3/2020- 7/17/2020": datetime(2020, 7, 3),
        "Not Dated Rec'd 4/22/2020": datetime(2020, 4, 22),
        "Not Dated Rec'd 4/13/2020": datetime(2020, 4, 13),
        "3/16 - 12/13/2020": datetime(2020, 3, 16),
        "february": None,
        "potentially": None,
        "not": None,
    }
    jobs_corrections = {
        "up to 703": 703,
        "18; 87": 105,
        "724 across U.S. including 49 from Ridgefield CT location": 49,
        "Not Provided": None,
        "Not Indicated": None,
        "Possibly 50+": 50,
        "Not indicated": None,
        "12; 6; 5": 23,
        "182; additional 21 on reduced hours": 182,
        "78; additional 13 on reduced hours": 78,
        "124; additional 30 on reduced hours": 124,
        "Not reported on WARN notice": None,
        "Not provided": None,
        "489 - total for CT and other locations": 489,
        "158 Stamford 81 Branford": 239,
        "?": None,
        "110 total; 7 CT 103 remote": 7,
        "Not": None,
        "208 (36 of whom work in CT)": 36,
        "No CT details provided": 0,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        try:
            dt = self.date_corrections[value]
            if dt:
                return str(dt.date())
            else:
                assert dt is None
                return dt
        except KeyError:
            pass
        value = value.lower()
        value = value.replace("beginning", "")
        value = value.replace("after", "")
        value = value.replace("estimated", "")
        value = value.replace(";", "")
        value = value.replace("*", "")
        value = value.strip().split()[0].strip()
        return super().transform_date(value)

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "yes" in row["closing"].lower() or None



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Georgia raw data for consolidation."""

    postal_code = "GA"
    fields = dict(
        company="Company Name",
        location=lambda row: row["First Location Address"] or row["County"],
        effective_date="First Date of Separation",
        jobs="Total Number of Affected Employees",
    )
    date_format = "%m/%d/%Y"

    # This needs to be checked -- no temporary data to test against
    def check_if_temporary(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a temporary or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "temporary" in row["Type of Layoff or Closure"].lower() or None

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "closure" in row["Type of Layoff or Closure"].lower() or None



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Deleware raw data for consolidation."""

    postal_code = "DE"
    fields = dict(
        company="employer",
        location="city",
        notice_date="notice_date",
        jobs="number_of_employees_affected",
    )
    date_format = "%b %d, %Y"



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Alaska raw data for consolidation."""

    postal_code = "AK"
    fields = dict(
        company="Company",
        location="Location",
        notice_date="Notice Date",
        effective_date="Layoff Date",
        jobs="Employees Affected",
    )
    date_format = "%m/%d/%y"
    date_corrections = {
        "9/30/20*": datetime(2020, 9, 30),
        "August-November 2021": datetime(2021, 8, 1),
        "4/1/20 5/31/20": datetime(2020, 4, 1),
        "Varied": None,
        "March to May 2016": datetime(2016, 3, 1),
        "various": None,
        "June-August 2023": datetime(2023, 6, 1),
        "9/6/2023": datetime(2023, 9, 6),
        "9/5/2023": datetime(2023, 9, 5),
    }
    jobs_corrections = {
        "Up to 300": 300,
        "TBA": None,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        try:
            dt = self.date_corrections[value]
            if dt:
                return str(dt.date())
            else:
                assert dt is None
                return dt
        except KeyError:
            pass
        value = value.strip()
        value = value.split(" to ")[0].strip()
        value = value.replace("Starting ", "").strip()
        return super().transform_date(value)

    def check_if_temporary(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a temporary or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "temporary" in row["Notes"].lower() or None

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "closure" in row["Notes"].lower() or None



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Utah raw data for consolidation."""

    postal_code = "UT"
    fields = dict(
        company="Company Name",
        location="Location",
        notice_date="Date of Notice",
        jobs="Affected Workers",
    )
    date_format = ("%m/%d/%Y", "%m/%d/%y")
    date_corrections = {
        "03/09/2020&": datetime(2020, 3, 9),
        "01/05/18/": datetime(2018, 1, 5),
        "03/05/14 Updated": datetime(2014, 3, 5),
        "09/31/10": datetime(2010, 9, 30),
        "05/2009": datetime(2009, 5, 1),
        "01/07//09": datetime(2009, 1, 7),
        "08/31//2022": datetime(2022, 8, 31),
    }
    jobs_corrections = {
        "645 Revised": 645,
    }



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Florida raw data for consolidation."""

    postal_code = "FL"
    fields = dict(
        company="Company Name",
        location="Company Name",
        notice_date="State Notification Date",
        effective_date="Layoff Date",
        jobs="Employees Affected",
    )
    date_format = ("%m-%d-%y", "%m/%d/%Y")
    jobs_corrections = {
        # This Disney layoff notice is large but legit
        # https://www.usatoday.com/story/travel/2020/10/30/disney-world-live-entertainment-shows-dark-covid-19-pandemic/6088586002/
        10903: 10903,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        try:
            return super().transform_date(value)
        except Exception:
            value = value.replace("\n", "").strip()
            value = value.split(" thru")[0].strip()
            value = value.split("thru")[0].strip()
            return super().transform_date(value)

    def transform_company(self, value: str) -> str:
        """Transform a raw company name.

        Args:
            value (str): The raw company string provided by the source

        Returns: A string object ready for consolidation.
        """
        return value.split("\n")[0].strip()

    def transform_location(self, value: str) -> str:
        """Transform a raw location.

        Args:
            value (str): The raw location string provided by the source

        Returns: A string object ready for consolidation.
        """
        return value.split("\n")[-1].split(",")[0].strip()



from pathlib import Path

import pytest

from warn_transformer import integrate


@pytest.mark.runvcr
@pytest.mark.vcr()
def test_integrate():
    """Test integrate."""
    this_dir = Path(__file__).parent
    new_path = this_dir / "data" / "processed" / "consolidated.csv"
    integrate.run(new_path, init_current_data=True)



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Illinois raw data for consolidation."""

    postal_code = "IL"
    fields = dict(
        company="Location Name",
        location=lambda row: f"{row['Location Address']} {row['Location City']}, {row['Location State']} {row['Location Zipcode']}".strip(),
        notice_date=lambda row: row["Initial Date Reported"]
        or row["Notification(s) Received"],
        effective_date="Impact Date",
        jobs="Revised Layoff",
    )
    date_format = "%Y-%m-%d %H:%M:%S"
    minimum_year = 1987
    maximum_jobs = 100000

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "closure" in row["Reason"].lower() or None



import os
import typing
from pathlib import Path

USER_DIR = Path(os.path.expanduser("~"))
DEFAULT_WARN_TRANSFORMER_OUTPUT_DIR = USER_DIR / ".warn-transformer"

WARN_TRANSFORMER_OUTPUT_DIR = Path(
    os.environ.get("WARN_TRANSFORMER_OUTPUT_DIR", DEFAULT_WARN_TRANSFORMER_OUTPUT_DIR)
)


def get_all_transformers() -> typing.List[str]:
    """Get all the states and territories that have scrapers.

    Returns: List of lower-case post abbreviations.
    """
    this_dir = Path(__file__).parent
    transformers_dir = this_dir / "transformers"
    return sorted(
        p.stem for p in transformers_dir.glob("*.py") if "__init__.py" not in str(p)
    )



import logging
import typing

from ..schema import BaseTransformer

logger = logging.getLogger(__name__)


class Transformer(BaseTransformer):
    """Transform Wisconsin raw data for consolidation."""

    postal_code = "WI"
    fields = dict(
        company="Company",
        location="City",
        notice_date="Notice Received",
        effective_date="Layoff Begin Date",
        jobs="Affected Workers",
    )
    date_format = (
        "%m/%d/%Y",
        "%Y%m%d",
        "%m/%d/%y",
    )
    date_corrections = {
        "11/03": None,
        "Unknown": None,
    }
    jobs_corrections = {
        "Unknown": None,
    }

    def transform_company(self, value: str) -> str:
        """Transform a raw company name.

        Args:
            value (str): The raw company string provided by the source

        Returns: A string object ready for consolidation.
        """
        # Cut revision notices
        value = value.split("- Revision")[0]
        # Do the typical stuff
        return super().transform_company(value)

    def check_if_amendment(self, row: typing.Dict) -> bool:
        """Determine whether a row is an amendment or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean
        """
        return "revision" in row["Company"].lower()

    def handle_amendments(
        self, row_list: typing.List[typing.Dict]
    ) -> typing.List[typing.Dict]:
        """Remove amended filings from the provided list of records.

        Args:
            row_list (list): A list of clean rows of data.

        Returns: A list of cleaned data, minus amended records.
        """
        amendments_count = len([r for r in row_list if r["is_amendment"] is True])
        logger.debug(f"{amendments_count} amendments in {self.postal_code}")

        # Loop through all the rows
        ancestor_list = []
        for i, row in enumerate(row_list):
            # If the current row is amended ...
            if row["is_amendment"]:
                # Get the previous record
                previous_index = i - 1
                previous_record = row_list[previous_index]

                # Are the first five characters of the company name found?
                similar_name = (
                    row["company"][:5].lower() in previous_record["company"].lower()
                )

                # If not, it's not an obvious ancestor
                if not similar_name:
                    logger.debug(f"No ancestor found for {row['company']}")
                    continue

                # If the name is similar, the previous record should be struck
                # We'll mark it by adding its hash_id to our list
                ancestor_list.append(previous_record["hash_id"])

        # Cut all of the ancestor records
        logger.debug(f"{len(ancestor_list)} ancestors removed")
        amended_list = [r for r in row_list if r["hash_id"] not in ancestor_list]

        # Return the rows we want to keep
        return amended_list



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Louisiana raw data for consolidation."""

    postal_code = "LA"
    fields = dict(
        company="Company Name",
        location="Location",
        notice_date="Notice Date",
        effective_date="Layoff Date",
        jobs="Employees Affected",
    )
    date_format = ["%m/%d/%Y", "%m/%d/%y"]
    date_corrections = {
        "6/31/09": datetime(2009, 6, 30),
        "N/A": None,
        "5/1820": datetime(2020, 5, 18),
        "Starting": datetime(2023, 8, 21),
        "4/10/23 (Updated 7/12/23)": datetime(2023, 4, 10),
        "2/26/25": datetime(2025, 2, 26),
        "12/31/25": datetime(2025, 12, 31),
        "Various": None,
    }
    jobs_corrections = {
        "700 *exact number pending relocation to other departments": 700,
        "50-297": 50,
        "426 426": 426,
        "1 Multi-state notification- Louisiana total =1": 1,
        "48 by 8/28/2015 closure by 12/2015": 48,
        "60-70": 60,
        "8 +1": 9,
        "385 465": 385,
        "114 +112": 226,
        "227 -8": 227,
        "32 +1": 32,
        "167 100": 167,
        "4 +55": 59,
        "150 +50": 200,
        "NA": None,
        "161 +1": 162,
        "23 15 41 4": 23,
        "70 +2": 72,
        "30 +5 +4 +1 +3": 43,
        "420 405 369": 420,
        "n/a": None,
        "74 +1 +21": 96,
        "84 -4 +10": 98,
        "1*": 1,
        "100 98": 98,
        "100 98 98": 98,
        "100 98 90": 90,
        "1 (Louisiana)": 1,
        "100 98 9087": 87,
        "100 98 90 87": 87,
        "TBD": None,
        "144 56": 56,
        "125* *Only one employee affected in Louisiana.": 1,
        "8*": 8,
        "51*": 51,
        "83*": 83,
        "38*": 38,
        "*292 *Only forty (40) employees affected in Louisiana.": 40,
        "*508 *Only sixty- three (63) employees affected in Louisiana.": 63,
        "*179 *Some or all of the workers may be retained by the new vendor.": 0,
        "*434 *Some or all may be picked up by new vendor": 0,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        # Try corrections before we edit the string
        try:
            dt = self.date_corrections[value]
            if dt:
                return str(dt.date())
            else:
                assert dt is None
                return dt
        except KeyError:
            pass

        # A little custom clean up based on the weird stuff from this source
        value = value.replace("starting", "")
        value = value.strip().split(" and ")[0].strip()
        value = value.strip().split(" to ")[0].strip()
        value = value.strip().split(" - ")[0].strip()
        value = value.strip().split(" & ")[0].strip()
        value = value.strip().split(" – ")[0].strip()
        value = value.strip().split("-")[0].strip()
        value = value.strip().split()[0].strip()

        # The same old stuff
        return super().transform_date(value)



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Montana raw data for consolidation."""

    postal_code = "MT"
    fields = dict(
        company="Name of Company",
        location="County",
        notice_date="Date of Notice",
        effective_date="Date of Impact",
        jobs="Number of Employees Affected",
    )
    date_format = ("%Y-%m-%d %H:%M:%S", "%Y-%m-%d")
    date_corrections = {
        "3/1620 to 4/30/20": datetime(2020, 3, 16),
        "Sept-April": None,
    }
    jobs_corrections = {
        "Not noted": None,
        "MT # unknown": None,
        "up to 300": 1,
        "Over 100": 100,
    }



#!/usr/bin/env python
"""Configure the package for distribution."""
import os

from setuptools import find_packages, setup


def read(file_name):
    """Read the provided file."""
    this_dir = os.path.dirname(__file__)
    file_path = os.path.join(this_dir, file_name)
    with open(file_path) as f:
        return f.read()


def version_scheme(version):
    """
    Version scheme hack for setuptools_scm.

    Appears to be necessary to due to the bug documented here: https://github.com/pypa/setuptools_scm/issues/342

    If that issue is resolved, this method can be removed.
    """
    import time

    from setuptools_scm.version import guess_next_version

    if version.exact:
        return version.format_with("{tag}")
    else:
        _super_value = version.format_next_version(guess_next_version)
        now = int(time.time())
        return _super_value + str(now)


def local_version(version):
    """
    Local version scheme hack for setuptools_scm.

    Appears to be necessary to due to the bug documented here: https://github.com/pypa/setuptools_scm/issues/342

    If that issue is resolved, this method can be removed.
    """
    return ""


setup(
    name="warn-transformer",
    description="Consolidate, enrich and analyze the data gathered by warn-scraper",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Big Local News",
    url="https://github.com/biglocalnews/warn-transformer",
    packages=find_packages(
        exclude=[
            "tests",
        ]
    ),
    include_package_data=True,
    install_requires=[
        "bln>=2.3.0",
        "marshmallow",
        "click",
        "jellyfish",
    ],
    license="Apache 2.0 license",
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    setup_requires=["setuptools_scm"],
    use_scm_version={"version_scheme": version_scheme, "local_scheme": local_version},
    entry_points="""
        [console_scripts]
        warn-transformer=warn_transformer.cli:cli
    """,
)



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Washington raw data for consolidation."""

    postal_code = "WA"
    fields = dict(
        company="Company",
        location="Location",
        notice_date="Layoff Start Date",
        effective_date="Layoff Start Date",
        jobs="# of Workers",
    )
    date_format = "%m/%d/%Y"

    def check_if_temporary(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a temporary or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return row["Type of Layoff"].lower() == "temporary"

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        temporary = row["Type of Layoff"].lower() == "temporary"
        if temporary:
            return False
        return row["Closure Layoff"].lower() == "closure"



import logging
import typing
from datetime import datetime

from ..schema import BaseTransformer

logger = logging.getLogger(__name__)


class Transformer(BaseTransformer):
    """Transform Iowa raw data for consolidation."""

    postal_code = "IA"
    fields = dict(
        company="Company",
        location="City",
        notice_date="Notice Date",
        effective_date="Layoff Date",
        jobs="Emp #",
    )
    date_format = ("%m/%d/%Y", "%Y-%m-%d %H:%M:%S")
    date_corrections = {
        "9/1/8/2020": datetime(2020, 1, 8),
        "4/26/21": datetime(2021, 4, 26),
        "2021-04-30 00:00:00": datetime(2021, 4, 30),
        "7/14/21": datetime(2021, 7, 14),
        "7/12/21": datetime(2021, 7, 12),
    }

    def check_if_amendment(self, row: typing.Dict) -> bool:
        """Determine whether a row is an amendment or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean
        """
        return "amendment" in row["Notice Type"].lower().strip()

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "closing" in row["Notice Type"].lower() or None

    def handle_amendments(
        self, row_list: typing.List[typing.Dict]
    ) -> typing.List[typing.Dict]:
        """Remove amended filings from the provided list of records.

        Args:
            row_list (list): A list of clean rows of data.

        Returns: A list of cleaned data, minus amended records.
        """
        amendments_count = len([r for r in row_list if r["is_amendment"] is True])
        logger.debug(f"{amendments_count} amendments in {self.postal_code}")
        logger.debug(
            "No action has been taken because it does not appear to be necessary to remove ancestor filings."
        )
        return row_list



import typing
from datetime import datetime

from ..schema import BaseTransformer


def transform_jobs(row):
    """Corrects variations in field names for the number of employees affected."""
    possible_headers = ["No. of EmployeesAffected", "No. of Employees Affected"]
    for field in possible_headers:
        try:
            return row[field]
        except KeyError:
            pass
    raise Exception(
        f"Unable to find the jobs data using these keys: {possible_headers}"
    )


class Transformer(BaseTransformer):
    """Transform Idaho raw data for consolidation."""

    postal_code = "ID"
    fields = dict(
        company="Company",
        location="City",
        notice_date="Date of Letter",
        effective_date="Effective or Commencing Date",
        # jobs="No. of EmployeesAffected",
        jobs=transform_jobs,
    )
    date_format = ("%m/%d/%Y", "%m/%d/%y")
    date_corrections = {
        "2/19/219": datetime(2019, 2, 19),
        "3/7/2010-3/20/2010": datetime(2010, 3, 7),
        "rting": datetime(2015, 2, 16),
        "3/7/2010-3/20/2": datetime(2010, 3, 7),
    }
    jobs_corrections = {
        "8 in ID": 8,
        "17 in ID": 17,
        "80-100": 80,
        "2 5s1ta": 251,
        "120 (2 in ID)": 2,
        "106 (17 in ID)": 17,
        22000: None,
        "22000 (102 in ID)": 102,
        "TBD": None,
        "135 (1 in ID)": 1,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        # A little custom clean up based on the weird stuff from this source
        if not value:
            value = ""
        value = value.replace("starting", "")
        value = value.strip().split()[0].replace(",", "").strip()

        # The same old stuff
        return super().transform_date(value)



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform California raw data for consolidation."""

    postal_code = "CA"
    fields = dict(
        company="company",
        location=lambda row: row["city"] or row["address"],
        notice_date="notice_date",
        effective_date="effective_date",
        jobs="num_employees",
    )
    date_format = "%m/%d/%Y"
    max_future_days = 365 * 5
    minimum_year = 2014
    jobs_corrections = {
        # This Tesla layoff number large but correct
        # https://www.cnbc.com/2020/05/13/coronavirus-latest-updates.html
        11083: 11083,
    }
    date_corrections = {
        "09/04/2008": datetime(2018, 9, 4),
        "07/04/2002": datetime(2020, 7, 4),
        "03/09/2121": datetime(2021, 3, 9),
        "03/30/3030": datetime(2020, 3, 30),
    }

    def check_if_temporary(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a temporary or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "temporary" in row["layoff_or_closure"].lower() or None

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return row["layoff_or_closure"].lower().strip() == "closure permanent" or None
